@@ -5,6 +5,8 @@ Note that PIL is not a dependency of SciPy and this module is not
 available on systems that don't have PIL installed.
 
 """
+from __future__ import division, print_function, absolute_import
+
 # Functions which need the PIL
 
 import numpy
@@ -14,34 +16,46 @@ from numpy import amin, amax, ravel, asarray, cast, arange, \
      ones, newaxis, transpose, mgrid, iscomplexobj, sum, zeros, uint8, \
      issubdtype, array
 
-import Image
-import ImageFilter
+try:
+    from PIL import Image, ImageFilter
+except ImportError:
+    import Image
+    import ImageFilter
+
+
+if not hasattr(Image, 'frombytes'):
+    Image.frombytes = Image.fromstring
 
 __all__ = ['fromimage','toimage','imsave','imread','bytescale',
-           'imrotate','imresize','imshow','imfilter','radon']
+           'imrotate','imresize','imshow','imfilter']
+
 
 # Returns a byte-scaled image
 def bytescale(data, cmin=None, cmax=None, high=255, low=0):
     """
     Byte scales an array (image).
 
+    Byte scaling means converting the input image to uint8 dtype and scaling
+    the range to ``(low, high)`` (default 0-255).
+    If the input image already has dtype uint8, no scaling is done.
+
     Parameters
     ----------
     data : ndarray
         PIL image data array.
-    cmin :  Scalar
-        Bias scaling of small values, Default is data.min().
-    cmax : scalar
-        Bias scaling of large values, Default is data.max().
-    high : scalar
-        Scale max value to `high`.
-    low : scalar
-        Scale min value to `low`.
+    cmin : scalar, optional
+        Bias scaling of small values. Default is ``data.min()``.
+    cmax : scalar, optional
+        Bias scaling of large values. Default is ``data.max()``.
+    high : scalar, optional
+        Scale max value to `high`.  Default is 255.
+    low : scalar, optional
+        Scale min value to `low`.  Default is 0.
 
     Returns
     -------
-    img_array : ndarray
-        Bytescaled array.
+    img_array : uint8 ndarray
+        The byte-scaled array.
 
     Examples
     --------
@@ -64,28 +78,43 @@ def bytescale(data, cmin=None, cmax=None, high=255, low=0):
     """
     if data.dtype == uint8:
         return data
-    high = high - low
-    if cmin is None: cmin = data.min()
-    if cmax is None: cmax = data.max()
-    scale = high *1.0 / (cmax-cmin or 1)
-    bytedata = ((data*1.0-cmin)*scale + 0.4999).astype(uint8)
-    return bytedata + cast[uint8](low)
 
-def imread(name,flatten=0):
+    if high < low:
+        raise ValueError("`high` should be larger than `low`.")
+
+    if cmin is None:
+        cmin = data.min()
+    if cmax is None:
+        cmax = data.max()
+
+    cscale = cmax - cmin
+    if cscale < 0:
+        raise ValueError("`cmax` should be larger than `cmin`.")
+    elif cscale == 0:
+        cscale = 1
+
+    scale = float(high - low) / cscale
+    bytedata = (data * 1.0 - cmin) * scale + 0.4999
+    bytedata[bytedata > high] = high
+    bytedata[bytedata < 0] = 0
+    return cast[uint8](bytedata) + cast[uint8](low)
+
+
+def imread(name, flatten=0):
     """
-    Read an image file from a filename.
+    Read an image from a file as an array.
 
     Parameters
     ----------
-    name : str
-        The file name to be read.
+    name : str or file object
+        The file name or file object to be read.
     flatten : bool, optional
         If True, flattens the color layers into a single gray-scale layer.
 
     Returns
     -------
-     : nd_array
-        The array obtained by reading image.
+    imread : ndarray
+        The array obtained by reading image from file `imfile`.
 
     Notes
     -----
@@ -97,30 +126,51 @@ def imread(name,flatten=0):
     im = Image.open(name)
     return fromimage(im,flatten=flatten)
 
-def imsave(name, arr):
+
+def imsave(name, arr, format=None):
     """
-    Save an array to an image file.
+    Save an array as an image.
 
     Parameters
     ----------
-    im : PIL image
-         Input image.
+    name : str or file object
+        Output file name or file object.
+    arr : ndarray, MxN or MxNx3 or MxNx4
+        Array containing image values.  If the shape is ``MxN``, the array
+        represents a grey-level image.  Shape ``MxNx3`` stores the red, green
+        and blue bands along the last dimension.  An alpha layer may be
+        included, specified as the last colour band of an ``MxNx4`` array.
+    format : str
+        Image format. If omitted, the format to use is determined from the 
+        file name extension. If a file object was used instead of a file name,
+        this parameter should always be used.
 
-    flatten : bool
-         If true, convert the output to grey-scale.
+    Examples
+    --------
+    Construct an array of gradient intensity values and save to file:
 
-    Returns
-    -------
-    img_array : ndarray
-                The different colour bands/channels are stored in the
-                third dimension, such that a grey-image is MxN, an
-                RGB-image MxNx3 and an RGBA-image MxNx4.
+    >>> x = np.zeros((255, 255))
+    >>> x = np.zeros((255, 255), dtype=np.uint8)
+    >>> x[:] = np.arange(255)
+    >>> imsave('/tmp/gradient.png', x)
+
+    Construct an array with three colour bands (R, G, B) and store to file:
+
+    >>> rgb = np.zeros((255, 255, 3), dtype=np.uint8)
+    >>> rgb[..., 0] = np.arange(255)
+    >>> rgb[..., 1] = 55
+    >>> rgb[..., 2] = 1 - np.arange(255)
+    >>> imsave('/tmp/rgb_gradient.png', rgb)
 
     """
     im = toimage(arr)
-    im.save(name)
+    if format is None:
+        im.save(name)
+    else:
+        im.save(name, format)
     return
 
+
 def fromimage(im, flatten=0):
     """
     Return a copy of a PIL image as a numpy array.
@@ -134,7 +184,7 @@ def fromimage(im, flatten=0):
 
     Returns
     -------
-    img_array : ndarray
+    fromimage : ndarray
         The different colour bands/channels are stored in the
         third dimension, such that a grey-image is MxN, an
         RGB-image MxNx3 and an RGBA-image MxNx4.
@@ -144,43 +194,54 @@ def fromimage(im, flatten=0):
         raise TypeError("Input is not a PIL image.")
     if flatten:
         im = im.convert('F')
+    elif im.mode == '1':
+        # workaround for crash in PIL, see #1613.
+        im.convert('L')
+
     return array(im)
 
 _errstr = "Mode is unknown or incompatible with input array shape."
-def toimage(arr,high=255,low=0,cmin=None,cmax=None,pal=None,
-            mode=None,channel_axis=None):
-    """Takes a numpy array and returns a PIL image.  The mode of the
-    PIL image depends on the array shape, the pal keyword, and the mode
-    keyword.
 
-    For 2-D arrays, if pal is a valid (N,3) byte-array giving the RGB values
-    (from 0 to 255) then mode='P', otherwise mode='L', unless mode is given
-    as 'F' or 'I' in which case a float and/or integer array is made
 
-    For 3-D arrays, the channel_axis argument tells which dimension of the
-      array holds the channel data.
+def toimage(arr, high=255, low=0, cmin=None, cmax=None, pal=None,
+            mode=None, channel_axis=None):
+    """Takes a numpy array and returns a PIL image.
+
+    The mode of the PIL image depends on the array shape and the `pal` and
+    `mode` keywords.
+
+    For 2-D arrays, if `pal` is a valid (N,3) byte-array giving the RGB values
+    (from 0 to 255) then ``mode='P'``, otherwise ``mode='L'``, unless mode
+    is given as 'F' or 'I' in which case a float and/or integer array is made.
+
+    Notes
+    -----
+    For 3-D arrays, the `channel_axis` argument tells which dimension of the
+    array holds the channel data.
+
     For 3-D arrays if one of the dimensions is 3, the mode is 'RGB'
-      by default or 'YCbCr' if selected.
-    if the
+    by default or 'YCbCr' if selected.
 
     The numpy array must be either 2 dimensional or 3 dimensional.
+
     """
     data = asarray(arr)
     if iscomplexobj(data):
         raise ValueError("Cannot convert a complex-valued array.")
     shape = list(data.shape)
-    valid = len(shape)==2 or ((len(shape)==3) and \
+    valid = len(shape) == 2 or ((len(shape) == 3) and
                               ((3 in shape) or (4 in shape)))
-    assert valid, "Not a suitable array shape for any mode."
+    if not valid:
+        raise ValueError("'arr' does not have a suitable array shape for any mode.")
     if len(shape) == 2:
-        shape = (shape[1],shape[0]) # columns show up first
+        shape = (shape[1],shape[0])  # columns show up first
         if mode == 'F':
             data32 = data.astype(numpy.float32)
-            image = Image.fromstring(mode,shape,data32.tostring())
+            image = Image.frombytes(mode,shape,data32.tostring())
             return image
         if mode in [None, 'L', 'P']:
             bytedata = bytescale(data,high=high,low=low,cmin=cmin,cmax=cmax)
-            image = Image.fromstring('L',shape,bytedata.tostring())
+            image = Image.frombytes('L',shape,bytedata.tostring())
             if pal is not None:
                 image.putpalette(asarray(pal,dtype=uint8).tostring())
                 # Becomes a mode='P' automagically.
@@ -191,7 +252,7 @@ def toimage(arr,high=255,low=0,cmin=None,cmax=None,pal=None,
             return image
         if mode == '1':  # high input gives threshold for 1
             bytedata = (data > high)
-            image = Image.fromstring('1',shape,bytedata.tostring())
+            image = Image.frombytes('1',shape,bytedata.tostring())
             return image
         if cmin is None:
             cmin = amin(ravel(data))
@@ -200,7 +261,7 @@ def toimage(arr,high=255,low=0,cmin=None,cmax=None,pal=None,
         data = (data*1.0 - cmin)*(high-low)/(cmax-cmin) + low
         if mode == 'I':
             data32 = data.astype(numpy.uint32)
-            image = Image.fromstring(mode,shape,data32.tostring())
+            image = Image.frombytes(mode,shape,data32.tostring())
         else:
             raise ValueError(_errstr)
         return image
@@ -234,50 +295,49 @@ def toimage(arr,high=255,low=0,cmin=None,cmax=None,pal=None,
         strdata = transpose(bytedata,(1,2,0)).tostring()
         shape = (shape[2],shape[1])
     if mode is None:
-        if numch == 3: mode = 'RGB'
-        else: mode = 'RGBA'
-
+        if numch == 3:
+            mode = 'RGB'
+        else:
+            mode = 'RGBA'
 
     if mode not in ['RGB','RGBA','YCbCr','CMYK']:
         raise ValueError(_errstr)
 
     if mode in ['RGB', 'YCbCr']:
-        assert numch == 3, "Invalid array shape for mode."
+        if numch != 3:
+            raise ValueError("Invalid array shape for mode.")
     if mode in ['RGBA', 'CMYK']:
-        assert numch == 4, "Invalid array shape for mode."
+        if numch != 4:
+            raise ValueError("Invalid array shape for mode.")
 
-    # Here we know data and mode is coorect
-    image = Image.fromstring(mode, shape, strdata)
+    # Here we know data and mode is correct
+    image = Image.frombytes(mode, shape, strdata)
     return image
 
+
 def imrotate(arr,angle,interp='bilinear'):
     """
     Rotate an image counter-clockwise by angle degrees.
 
     Parameters
     ----------
-    arr : nd_array
+    arr : ndarray
         Input array of image to be rotated.
     angle : float
         The angle of rotation.
     interp : str, optional
         Interpolation
 
+        - 'nearest' :  for nearest neighbor
+        - 'bilinear' : for bilinear
+        - 'cubic' : cubic
+        - 'bicubic' : for bicubic
 
     Returns
     -------
-     : nd_array
+    imrotate : ndarray
         The rotated array of image.
 
-    Notes
-    -----
-
-    Interpolation methods can be:
-    * 'nearest' :  for nearest neighbor
-    * 'bilinear' : for bilinear
-    * 'cubic' : cubic
-    * 'bicubic' : for bicubic
-
     """
     arr = asarray(arr)
     func = {'nearest':0,'bilinear':2,'bicubic':3,'cubic':3}
@@ -285,6 +345,7 @@ def imrotate(arr,angle,interp='bilinear'):
     im = im.rotate(angle,resample=func[interp])
     return fromimage(im)
 
+
 def imshow(arr):
     """
     Simple showing of an image through an external viewer.
@@ -326,13 +387,14 @@ def imshow(arr):
     if status != 0:
         raise RuntimeError('Could not execute image viewer.')
 
+
 def imresize(arr, size, interp='bilinear', mode=None):
     """
     Resize an image.
 
     Parameters
     ----------
-    arr : nd_array
+    arr : ndarray
         The array of image to be resized.
 
     size : int, float or tuple
@@ -340,23 +402,24 @@ def imresize(arr, size, interp='bilinear', mode=None):
         * float - Fraction of current size.
         * tuple - Size of the output image.
 
-    interp : string
-        interpolation to use for re-sizing ('nearest', 'bilinear', 'bicubic' or 'cubic')
+    interp : str
+        Interpolation to use for re-sizing ('nearest', 'bilinear', 'bicubic'
+        or 'cubic').
 
-    mode :
-        mode is the PIL image mode ('P', 'L', etc.)
+    mode : str
+        The PIL image mode ('P', 'L', etc.).
 
     Returns
     -------
-
-     : nd_array
+    imresize : ndarray
         The resized array of image.
 
     """
     im = toimage(arr, mode=mode)
     ts = type(size)
     if issubdtype(ts,int):
-        size = size / 100.0
+        percent = size / 100.0
+        size = (array(im.size)*percent).astype(int)
     elif issubdtype(type(size),float):
         size = (array(im.size)*size).astype(int)
     else:
@@ -381,13 +444,13 @@ def imfilter(arr,ftype):
 
     Returns
     -------
-    res : nd_array
+    imfilter : ndarray
         The array with filter applied.
 
     Raises
     ------
     ValueError
-        *Unknown filter type.* . If the filter you are trying
+        *Unknown filter type.*  If the filter you are trying
         to apply is unsupported.
 
     """
@@ -404,18 +467,6 @@ def imfilter(arr,ftype):
               }
 
     im = toimage(arr)
-    if ftype not in _tdict.keys():
+    if ftype not in _tdict:
         raise ValueError("Unknown filter type.")
     return fromimage(im.filter(_tdict[ftype]))
-
-
-def radon(arr,theta=None):
-    if theta is None:
-        theta = mgrid[0:180]
-    s = zeros((arr.shape[1],len(theta)), float)
-    k = 0
-    for th in theta:
-        im = imrotate(arr,-th)
-        s[:,k] = sum(im,axis=0)
-        k += 1
-    return s
