@@ -1,13 +1,15 @@
 from __future__ import division, print_function, absolute_import
 
-from numpy.testing import assert_, assert_array_almost_equal, assert_equal, \
-                          assert_almost_equal, assert_array_equal, \
-                          assert_raises, run_module_suite, TestCase
+import os.path
+import warnings
+
 import numpy as np
+from numpy.testing import (assert_, assert_array_almost_equal, assert_equal,
+                           assert_almost_equal, assert_array_equal,
+                           assert_raises, run_module_suite, TestCase)
 
 import scipy.ndimage as ndimage
 
-import os.path
 
 types = [np.int8, np.uint8, np.int16,
          np.uint16, np.int32, np.uint32,
@@ -117,49 +119,48 @@ class Test_measurements_select(TestCase):
 
 
 def test_label01():
-    "label 1"
     data = np.ones([])
     out, n = ndimage.label(data)
     assert_array_almost_equal(out, 1)
+    assert_equal(n, 1)
 
 
 def test_label02():
-    "label 2"
     data = np.zeros([])
     out, n = ndimage.label(data)
     assert_array_almost_equal(out, 0)
+    assert_equal(n, 0)
 
 
 def test_label03():
-    "label 3"
     data = np.ones([1])
     out, n = ndimage.label(data)
     assert_array_almost_equal(out, [1])
+    assert_equal(n, 1)
 
 
 def test_label04():
-    "label 4"
     data = np.zeros([1])
     out, n = ndimage.label(data)
     assert_array_almost_equal(out, [0])
+    assert_equal(n, 0)
 
 
 def test_label05():
-    "label 5"
     data = np.ones([5])
     out, n = ndimage.label(data)
     assert_array_almost_equal(out, [1, 1, 1, 1, 1])
+    assert_equal(n, 1)
 
 
 def test_label06():
-    "label 6"
     data = np.array([1, 0, 1, 1, 0, 1])
     out, n = ndimage.label(data)
     assert_array_almost_equal(out, [1, 0, 2, 2, 0, 3])
+    assert_equal(n, 3)
 
 
 def test_label07():
-    "label 7"
     data = np.array([[0, 0, 0, 0, 0, 0],
                            [0, 0, 0, 0, 0, 0],
                            [0, 0, 0, 0, 0, 0],
@@ -173,10 +174,10 @@ def test_label07():
                                [0, 0, 0, 0, 0, 0],
                                [0, 0, 0, 0, 0, 0],
                                [0, 0, 0, 0, 0, 0]])
+    assert_equal(n, 0)
 
 
 def test_label08():
-    "label 8"
     data = np.array([[1, 0, 0, 0, 0, 0],
                            [0, 0, 1, 1, 0, 0],
                            [0, 0, 1, 1, 1, 0],
@@ -190,10 +191,10 @@ def test_label08():
                                [3, 3, 0, 0, 0, 0],
                                [3, 3, 0, 0, 0, 0],
                                [0, 0, 0, 4, 4, 0]])
+    assert_equal(n, 4)
 
 
 def test_label09():
-    "label 9"
     data = np.array([[1, 0, 0, 0, 0, 0],
                            [0, 0, 1, 1, 0, 0],
                            [0, 0, 1, 1, 1, 0],
@@ -208,10 +209,10 @@ def test_label09():
                                [2, 2, 0, 0, 0, 0],
                                [2, 2, 0, 0, 0, 0],
                                [0, 0, 0, 3, 3, 0]])
+    assert_equal(n, 3)
 
 
 def test_label10():
-    "label 10"
     data = np.array([[0, 0, 0, 0, 0, 0],
                            [0, 1, 1, 0, 1, 0],
                            [0, 1, 1, 1, 1, 0],
@@ -222,10 +223,10 @@ def test_label10():
                                [0, 1, 1, 0, 1, 0],
                                [0, 1, 1, 1, 1, 0],
                                [0, 0, 0, 0, 0, 0]])
+    assert_equal(n, 1)
 
 
 def test_label11():
-    "label 11"
     for type in types:
         data = np.array([[1, 0, 0, 0, 0, 0],
                                [0, 0, 1, 1, 0, 0],
@@ -245,7 +246,6 @@ def test_label11():
 
 
 def test_label11_inplace():
-    "label 11 in place"
     for type in types:
         data = np.array([[1, 0, 0, 0, 0, 0],
                                [0, 0, 1, 1, 0, 0],
@@ -265,7 +265,6 @@ def test_label11_inplace():
 
 
 def test_label12():
-    "label 12"
     for type in types:
         data = np.array([[0, 0, 0, 0, 1, 1],
                                [0, 0, 0, 0, 0, 1],
@@ -283,7 +282,6 @@ def test_label12():
 
 
 def test_label13():
-    "label 13"
     for type in types:
         data = np.array([[1, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1],
                                [1, 1, 1, 0, 1, 1, 1, 0, 1, 1, 1],
@@ -300,7 +298,6 @@ def test_label13():
 
 
 def test_label_output_typed():
-    "test label with specified output with type"
     data = np.ones([5])
     for t in types:
         output = np.zeros([5], dtype=t)
@@ -310,7 +307,6 @@ def test_label_output_typed():
 
 
 def test_label_output_dtype():
-    "test label with specified output dtype"
     data = np.ones([5])
     for t in types:
         output, n = ndimage.label(data, output=t)
@@ -319,7 +315,6 @@ def test_label_output_dtype():
 
 
 def test_label_output_wrong_size():
-    "test label with output of wrong size"
     data = np.ones([5])
     for t in types:
         output = np.zeros([10], t)
@@ -327,7 +322,6 @@ def test_label_output_wrong_size():
 
 
 def test_label_structuring_elements():
-    "test label with different structuring element neighborhoods"
     data = np.loadtxt(os.path.join(os.path.dirname(__file__), "data", "label_inputs.txt"))
     strels = np.loadtxt(os.path.join(os.path.dirname(__file__), "data", "label_strels.txt"))
     results = np.loadtxt(os.path.join(os.path.dirname(__file__), "data", "label_results.txt"))
@@ -343,43 +337,45 @@ def test_label_structuring_elements():
             r += 1
 
 
+def test_label_default_dtype():
+    test_array = np.random.rand(10, 10)
+    label, no_features = ndimage.label(test_array > 0.5)
+    assert_(label.dtype in (np.int32, np.int64))
+    # Shouldn't raise an exception
+    ndimage.find_objects(label)
+
+
 def test_find_objects01():
-    "find_objects 1"
     data = np.ones([], dtype=int)
     out = ndimage.find_objects(data)
     assert_(out == [()])
 
 
 def test_find_objects02():
-    "find_objects 2"
     data = np.zeros([], dtype=int)
     out = ndimage.find_objects(data)
     assert_(out == [])
 
 
 def test_find_objects03():
-    "find_objects 3"
     data = np.ones([1], dtype=int)
     out = ndimage.find_objects(data)
     assert_equal(out, [(slice(0, 1, None),)])
 
 
 def test_find_objects04():
-    "find_objects 4"
     data = np.zeros([1], dtype=int)
     out = ndimage.find_objects(data)
     assert_equal(out, [])
 
 
 def test_find_objects05():
-    "find_objects 5"
     data = np.ones([5], dtype=int)
     out = ndimage.find_objects(data)
     assert_equal(out, [(slice(0, 5, None),)])
 
 
 def test_find_objects06():
-    "find_objects 6"
     data = np.array([1, 0, 2, 2, 0, 3])
     out = ndimage.find_objects(data)
     assert_equal(out, [(slice(0, 1, None),),
@@ -388,7 +384,6 @@ def test_find_objects06():
 
 
 def test_find_objects07():
-    "find_objects 7"
     data = np.array([[0, 0, 0, 0, 0, 0],
                         [0, 0, 0, 0, 0, 0],
                         [0, 0, 0, 0, 0, 0],
@@ -400,7 +395,6 @@ def test_find_objects07():
 
 
 def test_find_objects08():
-    "find_objects 8"
     data = np.array([[1, 0, 0, 0, 0, 0],
                            [0, 0, 2, 2, 0, 0],
                            [0, 0, 2, 2, 2, 0],
@@ -415,7 +409,6 @@ def test_find_objects08():
 
 
 def test_find_objects09():
-    "find_objects 9"
     data = np.array([[1, 0, 0, 0, 0, 0],
                            [0, 0, 2, 2, 0, 0],
                            [0, 0, 2, 2, 2, 0],
@@ -430,7 +423,6 @@ def test_find_objects09():
 
 
 def test_sum01():
-    "sum 1"
     for type in types:
         input = np.array([], type)
         output = ndimage.sum(input)
@@ -438,7 +430,6 @@ def test_sum01():
 
 
 def test_sum02():
-    "sum 2"
     for type in types:
         input = np.zeros([0, 4], type)
         output = ndimage.sum(input)
@@ -446,7 +437,6 @@ def test_sum02():
 
 
 def test_sum03():
-    "sum 3"
     for type in types:
         input = np.ones([], type)
         output = ndimage.sum(input)
@@ -454,7 +444,6 @@ def test_sum03():
 
 
 def test_sum04():
-    "sum 4"
     for type in types:
         input = np.array([1, 2], type)
         output = ndimage.sum(input)
@@ -462,7 +451,6 @@ def test_sum04():
 
 
 def test_sum05():
-    "sum 5"
     for type in types:
         input = np.array([[1, 2], [3, 4]], type)
         output = ndimage.sum(input)
@@ -470,7 +458,6 @@ def test_sum05():
 
 
 def test_sum06():
-    "sum 6"
     labels = np.array([], bool)
     for type in types:
         input = np.array([], type)
@@ -479,7 +466,6 @@ def test_sum06():
 
 
 def test_sum07():
-    "sum 7"
     labels = np.ones([0, 4], bool)
     for type in types:
         input = np.zeros([0, 4], type)
@@ -488,7 +474,6 @@ def test_sum07():
 
 
 def test_sum08():
-    "sum 8"
     labels = np.array([1, 0], bool)
     for type in types:
         input = np.array([1, 2], type)
@@ -497,7 +482,6 @@ def test_sum08():
 
 
 def test_sum09():
-    "sum 9"
     labels = np.array([1, 0], bool)
     for type in types:
         input = np.array([[1, 2], [3, 4]], type)
@@ -506,7 +490,6 @@ def test_sum09():
 
 
 def test_sum10():
-    "sum 10"
     labels = np.array([1, 0], bool)
     input = np.array([[1, 2], [3, 4]], bool)
     output = ndimage.sum(input, labels=labels)
@@ -514,7 +497,6 @@ def test_sum10():
 
 
 def test_sum11():
-    "sum 11"
     labels = np.array([1, 2], np.int8)
     for type in types:
         input = np.array([[1, 2], [3, 4]], type)
@@ -524,7 +506,6 @@ def test_sum11():
 
 
 def test_sum12():
-    "sum 12"
     labels = np.array([[1, 2], [2, 4]], np.int8)
     for type in types:
         input = np.array([[1, 2], [3, 4]], type)
@@ -534,7 +515,6 @@ def test_sum12():
 
 
 def test_mean01():
-    "mean 1"
     labels = np.array([1, 0], bool)
     for type in types:
         input = np.array([[1, 2], [3, 4]], type)
@@ -543,7 +523,6 @@ def test_mean01():
 
 
 def test_mean02():
-    "mean 2"
     labels = np.array([1, 0], bool)
     input = np.array([[1, 2], [3, 4]], bool)
     output = ndimage.mean(input, labels=labels)
@@ -551,7 +530,6 @@ def test_mean02():
 
 
 def test_mean03():
-    "mean 3"
     labels = np.array([1, 2])
     for type in types:
         input = np.array([[1, 2], [3, 4]], type)
@@ -561,7 +539,6 @@ def test_mean03():
 
 
 def test_mean04():
-    "mean 4"
     labels = np.array([[1, 2], [2, 4]], np.int8)
     olderr = np.seterr(all='ignore')
     try:
@@ -576,7 +553,6 @@ def test_mean04():
 
 
 def test_minimum01():
-    "minimum 1"
     labels = np.array([1, 0], bool)
     for type in types:
         input = np.array([[1, 2], [3, 4]], type)
@@ -585,7 +561,6 @@ def test_minimum01():
 
 
 def test_minimum02():
-    "minimum 2"
     labels = np.array([1, 0], bool)
     input = np.array([[2, 2], [2, 4]], bool)
     output = ndimage.minimum(input, labels=labels)
@@ -593,7 +568,6 @@ def test_minimum02():
 
 
 def test_minimum03():
-    "minimum 3"
     labels = np.array([1, 2])
     for type in types:
         input = np.array([[1, 2], [3, 4]], type)
@@ -603,7 +577,6 @@ def test_minimum03():
 
 
 def test_minimum04():
-    "minimum 4"
     labels = np.array([[1, 2], [2, 3]])
     for type in types:
         input = np.array([[1, 2], [3, 4]], type)
@@ -613,7 +586,6 @@ def test_minimum04():
 
 
 def test_maximum01():
-    "maximum 1"
     labels = np.array([1, 0], bool)
     for type in types:
         input = np.array([[1, 2], [3, 4]], type)
@@ -622,7 +594,6 @@ def test_maximum01():
 
 
 def test_maximum02():
-    "maximum 2"
     labels = np.array([1, 0], bool)
     input = np.array([[2, 2], [2, 4]], bool)
     output = ndimage.maximum(input, labels=labels)
@@ -630,7 +601,6 @@ def test_maximum02():
 
 
 def test_maximum03():
-    "maximum 3"
     labels = np.array([1, 2])
     for type in types:
         input = np.array([[1, 2], [3, 4]], type)
@@ -640,7 +610,6 @@ def test_maximum03():
 
 
 def test_maximum04():
-    "maximum 4"
     labels = np.array([[1, 2], [2, 3]])
     for type in types:
         input = np.array([[1, 2], [3, 4]], type)
@@ -650,13 +619,12 @@ def test_maximum04():
 
 
 def test_maximum05():
-    "Ticket #501"
+    # Regression test for ticket #501 (Trac)
     x = np.array([-3,-2,-1])
     assert_equal(ndimage.maximum(x),-1)
 
 
 def test_median01():
-    "median 1"
     a = np.array([[1, 2, 0, 1],
                   [5, 3, 0, 4],
                   [0, 0, 0, 7],
@@ -670,7 +638,6 @@ def test_median01():
 
 
 def test_median02():
-    "median 2"
     a = np.array([[1, 2, 0, 1],
                   [5, 3, 0, 4],
                   [0, 0, 0, 7],
@@ -680,7 +647,6 @@ def test_median02():
 
 
 def test_median03():
-    "median 3"
     a = np.array([[1, 2, 0, 1],
                   [5, 3, 0, 4],
                   [0, 0, 0, 7],
@@ -694,19 +660,20 @@ def test_median03():
 
 
 def test_variance01():
-    "variance 1"
     olderr = np.seterr(all='ignore')
     try:
-        for type in types:
-            input = np.array([], type)
-            output = ndimage.variance(input)
-            assert_(np.isnan(output))
+        with warnings.catch_warnings():
+            # Numpy 1.9 gives warnings for mean([])
+            warnings.filterwarnings('ignore', message="Mean of empty slice.")
+            for type in types:
+                input = np.array([], type)
+                output = ndimage.variance(input)
+                assert_(np.isnan(output))
     finally:
         np.seterr(**olderr)
 
 
 def test_variance02():
-    "variance 2"
     for type in types:
         input = np.array([1], type)
         output = ndimage.variance(input)
@@ -714,7 +681,6 @@ def test_variance02():
 
 
 def test_variance03():
-    "variance 3"
     for type in types:
         input = np.array([1, 3], type)
         output = ndimage.variance(input)
@@ -722,14 +688,12 @@ def test_variance03():
 
 
 def test_variance04():
-    "variance 4"
     input = np.array([1, 0], bool)
     output = ndimage.variance(input)
     assert_almost_equal(output, 0.25)
 
 
 def test_variance05():
-    "variance 5"
     labels = [2, 2, 3]
     for type in types:
         input = np.array([1, 3, 8], type)
@@ -738,7 +702,6 @@ def test_variance05():
 
 
 def test_variance06():
-    "variance 6"
     labels = [2, 2, 3, 3, 4]
     olderr = np.seterr(all='ignore')
     try:
@@ -751,19 +714,20 @@ def test_variance06():
 
 
 def test_standard_deviation01():
-    "standard deviation 1"
     olderr = np.seterr(all='ignore')
     try:
-        for type in types:
-            input = np.array([], type)
-            output = ndimage.standard_deviation(input)
-            assert_(np.isnan(output))
+        with warnings.catch_warnings():
+            # Numpy 1.9 gives warnings for mean([])
+            warnings.filterwarnings('ignore', message="Mean of empty slice.")
+            for type in types:
+                input = np.array([], type)
+                output = ndimage.standard_deviation(input)
+                assert_(np.isnan(output))
     finally:
         np.seterr(**olderr)
 
 
 def test_standard_deviation02():
-    "standard deviation 2"
     for type in types:
         input = np.array([1], type)
         output = ndimage.standard_deviation(input)
@@ -771,7 +735,6 @@ def test_standard_deviation02():
 
 
 def test_standard_deviation03():
-    "standard deviation 3"
     for type in types:
         input = np.array([1, 3], type)
         output = ndimage.standard_deviation(input)
@@ -779,14 +742,12 @@ def test_standard_deviation03():
 
 
 def test_standard_deviation04():
-    "standard deviation 4"
     input = np.array([1, 0], bool)
     output = ndimage.standard_deviation(input)
     assert_almost_equal(output, 0.5)
 
 
 def test_standard_deviation05():
-    "standard deviation 5"
     labels = [2, 2, 3]
     for type in types:
         input = np.array([1, 3, 8], type)
@@ -795,7 +756,6 @@ def test_standard_deviation05():
 
 
 def test_standard_deviation06():
-    "standard deviation 6"
     labels = [2, 2, 3, 3, 4]
     olderr = np.seterr(all='ignore')
     try:
@@ -808,7 +768,6 @@ def test_standard_deviation06():
 
 
 def test_standard_deviation07():
-    "standard deviation 7"
     labels = [1]
     olderr = np.seterr(all='ignore')
     try:
@@ -821,7 +780,6 @@ def test_standard_deviation07():
 
 
 def test_minimum_position01():
-    "minimum position 1"
     labels = np.array([1, 0], bool)
     for type in types:
         input = np.array([[1, 2], [3, 4]], type)
@@ -830,7 +788,6 @@ def test_minimum_position01():
 
 
 def test_minimum_position02():
-    "minimum position 2"
     for type in types:
         input = np.array([[5, 4, 2, 5],
                                 [3, 7, 0, 2],
@@ -840,7 +797,6 @@ def test_minimum_position02():
 
 
 def test_minimum_position03():
-    "minimum position 3"
     input = np.array([[5, 4, 2, 5],
                             [3, 7, 0, 2],
                             [1, 5, 1, 1]], bool)
@@ -849,7 +805,6 @@ def test_minimum_position03():
 
 
 def test_minimum_position04():
-    "minimum position 4"
     input = np.array([[5, 4, 2, 5],
                             [3, 7, 1, 2],
                             [1, 5, 1, 1]], bool)
@@ -858,7 +813,6 @@ def test_minimum_position04():
 
 
 def test_minimum_position05():
-    "minimum position 5"
     labels = [1, 2, 0, 4]
     for type in types:
         input = np.array([[5, 4, 2, 5],
@@ -869,7 +823,6 @@ def test_minimum_position05():
 
 
 def test_minimum_position06():
-    "minimum position 6"
     labels = [1, 2, 3, 4]
     for type in types:
         input = np.array([[5, 4, 2, 5],
@@ -880,7 +833,6 @@ def test_minimum_position06():
 
 
 def test_minimum_position07():
-    "minimum position 7"
     labels = [1, 2, 3, 4]
     for type in types:
         input = np.array([[5, 4, 2, 5],
@@ -893,7 +845,6 @@ def test_minimum_position07():
 
 
 def test_maximum_position01():
-    "maximum position 1"
     labels = np.array([1, 0], bool)
     for type in types:
         input = np.array([[1, 2], [3, 4]], type)
@@ -903,7 +854,6 @@ def test_maximum_position01():
 
 
 def test_maximum_position02():
-    "maximum position 2"
     for type in types:
         input = np.array([[5, 4, 2, 5],
                                 [3, 7, 8, 2],
@@ -913,7 +863,6 @@ def test_maximum_position02():
 
 
 def test_maximum_position03():
-    "maximum position 3"
     input = np.array([[5, 4, 2, 5],
                             [3, 7, 8, 2],
                             [1, 5, 1, 1]], bool)
@@ -922,7 +871,6 @@ def test_maximum_position03():
 
 
 def test_maximum_position04():
-    "maximum position 4"
     labels = [1, 2, 0, 4]
     for type in types:
         input = np.array([[5, 4, 2, 5],
@@ -933,7 +881,6 @@ def test_maximum_position04():
 
 
 def test_maximum_position05():
-    "maximum position 5"
     labels = [1, 2, 0, 4]
     for type in types:
         input = np.array([[5, 4, 2, 5],
@@ -944,7 +891,6 @@ def test_maximum_position05():
 
 
 def test_maximum_position06():
-    "maximum position 6"
     labels = [1, 2, 0, 4]
     for type in types:
         input = np.array([[5, 4, 2, 5],
@@ -957,7 +903,7 @@ def test_maximum_position06():
 
 
 def test_maximum_position07():
-    "maximum position 7 - float labels"
+    # Test float labels
     labels = np.array([1.0, 2.5, 0.0, 4.5])
     for type in types:
         input = np.array([[5, 4, 2, 5],
@@ -970,7 +916,6 @@ def test_maximum_position07():
 
 
 def test_extrema01():
-    "extrema 1"
     labels = np.array([1, 0], bool)
     for type in types:
         input = np.array([[1, 2], [3, 4]], type)
@@ -985,7 +930,6 @@ def test_extrema01():
 
 
 def test_extrema02():
-    "extrema 2"
     labels = np.array([1, 2])
     for type in types:
         input = np.array([[1, 2], [3, 4]], type)
@@ -1003,7 +947,6 @@ def test_extrema02():
 
 
 def test_extrema03():
-    "extrema 3"
     labels = np.array([[1, 2], [2, 3]])
     for type in types:
         input = np.array([[1, 2], [3, 4]], type)
@@ -1024,7 +967,6 @@ def test_extrema03():
 
 
 def test_extrema04():
-    "extrema 4"
     labels = [1, 2, 0, 4]
     for type in types:
         input = np.array([[5, 4, 2, 5],
@@ -1044,7 +986,6 @@ def test_extrema04():
 
 
 def test_center_of_mass01():
-    "center of mass 1"
     expected = [0.0, 0.0]
     for type in types:
         input = np.array([[1, 0], [0, 0]], type)
@@ -1053,7 +994,6 @@ def test_center_of_mass01():
 
 
 def test_center_of_mass02():
-    "center of mass 2"
     expected = [1, 0]
     for type in types:
         input = np.array([[0, 0], [1, 0]], type)
@@ -1062,7 +1002,6 @@ def test_center_of_mass02():
 
 
 def test_center_of_mass03():
-    "center of mass 3"
     expected = [0, 1]
     for type in types:
         input = np.array([[0, 1], [0, 0]], type)
@@ -1071,7 +1010,6 @@ def test_center_of_mass03():
 
 
 def test_center_of_mass04():
-    "center of mass 4"
     expected = [1, 1]
     for type in types:
         input = np.array([[0, 0], [0, 1]], type)
@@ -1080,7 +1018,6 @@ def test_center_of_mass04():
 
 
 def test_center_of_mass05():
-    "center of mass 5"
     expected = [0.5, 0.5]
     for type in types:
         input = np.array([[1, 1], [1, 1]], type)
@@ -1089,7 +1026,6 @@ def test_center_of_mass05():
 
 
 def test_center_of_mass06():
-    "center of mass 6"
     expected = [0.5, 0.5]
     input = np.array([[1, 2], [3, 1]], bool)
     output = ndimage.center_of_mass(input)
@@ -1097,7 +1033,6 @@ def test_center_of_mass06():
 
 
 def test_center_of_mass07():
-    "center of mass 7"
     labels = [1, 0]
     expected = [0.5, 0.0]
     input = np.array([[1, 2], [3, 1]], bool)
@@ -1106,7 +1041,6 @@ def test_center_of_mass07():
 
 
 def test_center_of_mass08():
-    "center of mass 8"
     labels = [1, 2]
     expected = [0.5, 1.0]
     input = np.array([[5, 2], [3, 1]], bool)
@@ -1115,7 +1049,6 @@ def test_center_of_mass08():
 
 
 def test_center_of_mass09():
-    "center of mass 9"
     labels = [1, 2]
     expected = [(0.5, 0.0), (0.5, 1.0)]
     input = np.array([[1, 2], [1, 1]], bool)
@@ -1124,7 +1057,6 @@ def test_center_of_mass09():
 
 
 def test_histogram01():
-    "histogram 1"
     expected = np.ones(10)
     input = np.arange(10)
     output = ndimage.histogram(input, 0, 10, 10)
@@ -1132,7 +1064,6 @@ def test_histogram01():
 
 
 def test_histogram02():
-    "histogram 2"
     labels = [1, 1, 1, 1, 2, 2, 2, 2]
     expected = [0, 2, 0, 1, 1]
     input = np.array([1, 1, 3, 4, 3, 3, 3, 3])
@@ -1141,7 +1072,6 @@ def test_histogram02():
 
 
 def test_histogram03():
-    "histogram 3"
     labels = [1, 0, 1, 1, 2, 2, 2, 2]
     expected1 = [0, 1, 0, 1, 1]
     expected2 = [0, 0, 0, 3, 0]
@@ -1153,7 +1083,6 @@ def test_histogram03():
 
 
 def test_stat_funcs_2d():
-    """Apply the stat funcs to a 2-d array."""
     a = np.array([[5,6,0,0,0], [8,9,0,0,0], [0,0,0,3,5]])
     lbl = np.array([[1,1,0,0,0], [1,1,0,0,0], [0,0,0,2,2]])
 
