@@ -1,11 +1,14 @@
+from __future__ import absolute_import, print_function
+
 import os
 import sys
 import re
 
-import catalog
-import build_tools
-import converters
-import base_spec
+from . import catalog
+from . import build_tools
+from . import converters
+from . import base_spec
+
 
 class ext_function_from_specs(object):
     def __init__(self,name,code_block,arg_specs):
@@ -19,7 +22,7 @@ class ext_function_from_specs(object):
         pass
 
     def function_declaration_code(self):
-        code  = 'static PyObject* %s(PyObject*self, PyObject* args,' \
+        code = 'static PyObject* %s(PyObject*self, PyObject* args,' \
                 ' PyObject* kywds)\n{\n'
         return code % self.name
 
@@ -42,11 +45,12 @@ class ext_function_from_specs(object):
         """
 
         declare_return = 'py::object return_val;\n' \
-                         'int exception_occured = 0;\n' \
+                         'int exception_occurred = 0;\n' \
                          'PyObject *py_local_dict = NULL;\n'
         arg_string_list = self.arg_specs.variable_as_strings() + ['"local_dict"']
         arg_strings = ','.join(arg_string_list)
-        if arg_strings: arg_strings += ','
+        if arg_strings:
+            arg_strings += ','
         declare_kwlist = 'static const char *kwlist[] = {%s NULL};\n' % \
                          arg_strings
 
@@ -55,9 +59,9 @@ class ext_function_from_specs(object):
         init_flags_init = '= '.join(self.arg_specs.init_flags())
         py_vars = ' = '.join(self.arg_specs.py_variables())
         if py_objects:
-            declare_py_objects  = 'PyObject ' + py_objects +';\n'
-            declare_py_objects += 'int '+ init_flags + ';\n'
-            init_values  = py_vars + ' = NULL;\n'
+            declare_py_objects = 'PyObject ' + py_objects + ';\n'
+            declare_py_objects += 'int ' + init_flags + ';\n'
+            init_values = py_vars + ' = NULL;\n'
             init_values += init_flags_init + ' = 0;\n\n'
         else:
             declare_py_objects = ''
@@ -73,20 +77,20 @@ class ext_function_from_specs(object):
         else:
             ref_string = '&py_local_dict'
 
-        format = "O"* len(self.arg_specs) + "|O" + ':' + self.name
-        parse_tuple =  'if(!PyArg_ParseTupleAndKeywords(args,' \
+        format = "O" * len(self.arg_specs) + "|O" + ':' + self.name
+        parse_tuple = 'if(!PyArg_ParseTupleAndKeywords(args,' \
                              'kywds,"%s",const_cast<char**>(kwlist),%s))\n' % \
                              (format,ref_string)
         parse_tuple += '   return NULL;\n'
 
-        return   declare_return + declare_kwlist + declare_py_objects  \
+        return declare_return + declare_kwlist + declare_py_objects  \
                + init_values + parse_tuple
 
     def arg_declaration_code(self):
         arg_strings = []
         for arg in self.arg_specs:
             arg_strings.append(arg.declaration_code())
-            arg_strings.append(arg.init_flag() +" = 1;\n")
+            arg_strings.append(arg.init_flag() + " = 1;\n")
         code = "".join(arg_strings)
         return code
 
@@ -94,9 +98,9 @@ class ext_function_from_specs(object):
         arg_strings = []
         have_cleanup = filter(lambda x:x.cleanup_code(),self.arg_specs)
         for arg in have_cleanup:
-            code  = "if(%s)\n" % arg.init_flag()
+            code = "if(%s)\n" % arg.init_flag()
             code += "{\n"
-            code +=     indent(arg.cleanup_code(),4)
+            code += indent(arg.cleanup_code(),4)
             code += "}\n"
             arg_strings.append(code)
         code = "".join(arg_strings)
@@ -118,35 +122,35 @@ class ext_function_from_specs(object):
         dict_code = "if(py_local_dict)                                  \n"   \
                     "{                                                  \n"   \
                     "    py::dict local_dict = py::dict(py_local_dict); \n" + \
-                         local_dict_code                                    + \
+                         local_dict_code + \
                     "}                                                  \n"
 
-        try_code =    "try                              \n"   \
+        try_code = "try                              \n"   \
                       "{                                \n" + \
-                           decl_code                        + \
+                           decl_code + \
                       "    /*<function call here>*/     \n" + \
-                           function_code                    + \
-                           indent(dict_code,4)              + \
+                           function_code + \
+                           indent(dict_code,4) + \
                       "\n}                                \n"
-        catch_code =  "catch(...)                       \n"   \
+        catch_code = "catch(...)                       \n"   \
                       "{                                \n" + \
                       "    return_val =  py::object();      \n"   \
-                      "    exception_occured = 1;       \n"   \
+                      "    exception_occurred = 1;       \n"   \
                       "}                                \n"
 
         return_code = "    /*cleanup code*/                     \n" + \
-                           cleanup_code                             + \
-                      '    if(!(PyObject*)return_val && !exception_occured)\n'   \
+                           cleanup_code + \
+                      '    if(!(PyObject*)return_val && !exception_occurred)\n'   \
                       '    {\n                                  \n'   \
                       '        return_val = Py_None;            \n'   \
                       '    }\n                                  \n'   \
                       '    return return_val.disown();           \n'           \
                       '}                                \n'
 
-        all_code = self.function_declaration_code()         + \
-                       indent(self.parse_tuple_code(),4)    + \
-                       indent(try_code,4)                   + \
-                       indent(catch_code,4)                 + \
+        all_code = self.function_declaration_code() + \
+                       indent(self.parse_tuple_code(),4) + \
+                       indent(try_code,4) + \
+                       indent(catch_code,4) + \
                        return_code
 
         return all_code
@@ -179,7 +183,8 @@ class ext_function(ext_function_from_specs):
         ext_function_from_specs.__init__(self,name,code_block,arg_specs)
 
 
-import base_info
+from . import base_info
+
 
 class ext_module(object):
     def __init__(self,name,compiler=''):
@@ -192,6 +197,7 @@ class ext_module(object):
 
     def add_function(self,func):
         self.functions.append(func)
+
     def module_code(self):
         code = '\n'.join([
             """\
@@ -253,12 +259,12 @@ extern "C" {
 
     def warning_code(self):
         all_warnings = self.build_information().warnings()
-        w=map(lambda x: "#pragma warning(%s)\n" % x,all_warnings)
+        w = map(lambda x: "#pragma warning(%s)\n" % x,all_warnings)
         return '#ifndef __GNUC__\n' + ''.join(w) + '\n#endif'
 
     def header_code(self):
         h = self.get_headers()
-        h= map(lambda x: '#include ' + x + '\n',h)
+        h = map(lambda x: '#include ' + x + '\n',h)
         return ''.join(h) + '\n'
 
     def support_code(self):
@@ -275,7 +281,7 @@ extern "C" {
         all_definition_code = ""
         for func in self.functions:
             all_definition_code += func.python_function_definition_code()
-        all_definition_code =  indent(''.join(all_definition_code),4)
+        all_definition_code = indent(''.join(all_definition_code),4)
         code = 'static PyMethodDef compiled_methods[] = \n' \
                '{\n' \
                '%s' \
@@ -284,7 +290,7 @@ extern "C" {
         return code % (all_definition_code)
 
     def module_init_code(self):
-        init_code_list =  self.build_information().module_init_code()
+        init_code_list = self.build_information().module_init_code()
         init_code = indent(''.join(init_code_list),4)
         code = 'PyMODINIT_FUNC init%s(void)\n' \
                '{\n' \
@@ -340,7 +346,7 @@ extern "C" {
         kw,file = self.build_kw_and_file(location,kw)
         return build_tools.create_extension(file, **kw)
 
-    def compile(self,location='.',compiler=None, verbose = 0, **kw):
+    def compile(self,location='.',compiler=None, verbose=0, **kw):
 
         if compiler is not None:
             self.compiler = compiler
@@ -362,16 +368,18 @@ extern "C" {
         # for speed, build in the machines temp directory
         temp = catalog.intermediate_dir()
 
-        success = build_tools.build_extension(file, temp_dir = temp,
-                                              compiler_name = compiler,
-                                              verbose = verbose, **kw)
+        success = build_tools.build_extension(file, temp_dir=temp,
+                                              compiler_name=compiler,
+                                              verbose=verbose, **kw)
         if not success:
             raise SystemError('Compilation failed')
 
+
 def generate_file_name(module_name,module_location):
     module_file = os.path.join(module_location,module_name)
     return os.path.abspath(module_file)
 
+
 def generate_module(module_string, module_file):
     """ generate the source code file.  Only overwrite
         the existing file if the actual source has changed.
@@ -389,14 +397,15 @@ def generate_module(module_string, module_file):
         f.close()
     return module_file
 
-def assign_variable_types(variables,local_dict = {}, global_dict = {},
-                          auto_downcast = 1,
-                          type_converters = converters.default):
+
+def assign_variable_types(variables,local_dict={}, global_dict={},
+                          auto_downcast=1,
+                          type_converters=converters.default):
     incoming_vars = {}
     incoming_vars.update(global_dict)
     incoming_vars.update(local_dict)
     variable_specs = []
-    errors={}
+    errors = {}
     for var in variables:
         try:
             example_type = incoming_vars[var]
@@ -417,7 +426,7 @@ def assign_variable_types(variables,local_dict = {}, global_dict = {},
             errors[var] = ("The type and dimensionality specifications" +
                            "for variable '" + var + "' are missing.")
         except IndexError:
-            errors[var] = ("Unable to convert variable '"+ var +
+            errors[var] = ("Unable to convert variable '" + var +
                            "' to a C++ type.")
     if errors:
         raise TypeError(format_error_msg(errors))
@@ -426,6 +435,7 @@ def assign_variable_types(variables,local_dict = {}, global_dict = {},
         variable_specs = downcast(variable_specs)
     return variable_specs
 
+
 def downcast(var_specs):
     """ Cast python scalars down to most common type of
          arrays used.
@@ -444,8 +454,8 @@ def downcast(var_specs):
 
     # if arrays are present, but none of them are double precision,
     # make all numeric types float or complex(float)
-    if (    ('f' in numeric_types or 'F' in numeric_types) and
-        not ('d' in numeric_types or 'D' in numeric_types) ):
+    if (('f' in numeric_types or 'F' in numeric_types) and
+        not ('d' in numeric_types or 'D' in numeric_types)):
         for var in var_specs:
             if hasattr(var,'numeric_type'):
                 if issubclass(var.numeric_type, complex):
@@ -454,6 +464,7 @@ def downcast(var_specs):
                     var.numeric_type = 'f'
     return var_specs
 
+
 def indent(st,spaces):
     indention = ' '*spaces
     indented = indention + st.replace('\n','\n'+indention)
@@ -461,9 +472,11 @@ def indent(st,spaces):
     indented = re.sub(r' +$',r'',indented)
     return indented
 
+
 def format_error_msg(errors):
     #minimum effort right now...
-    import pprint,cStringIO
+    import pprint
+    import cStringIO
     msg = cStringIO.StringIO()
     pprint.pprint(errors,msg)
     return msg.getvalue()
